@@ -1,7 +1,7 @@
 /*
  * query.c -- nsd(8) the resolver.
  *
- * Copyright (c) 2001-2011, NLnet Labs. All rights reserved.
+ * Copyright (c) 2001-2006, NLnet Labs. All rights reserved.
  *
  * See LICENSE for the license.
  *
@@ -54,14 +54,14 @@ static int add_rrset(struct query  *query,
 static void answer_authoritative(struct nsd	  *nsd,
 				 struct query     *q,
 				 answer_type      *answer,
-				 uint32_t          domain_number,
+				 size_t            domain_number,
 				 int               exact,
 				 domain_type      *closest_match,
 				 domain_type      *closest_encloser,
 				 const dname_type *qname);
 
 static void answer_lookup_zone(struct nsd *nsd, struct query *q,
-			       answer_type *answer, uint32_t domain_number,
+			       answer_type *answer, size_t domain_number,
 			       int exact, domain_type *closest_match,
 			       domain_type *closest_encloser,
 			       const dname_type *qname);
@@ -113,7 +113,7 @@ query_add_compression_domain(struct query *q, domain_type *domain, uint16_t offs
 	while (domain->parent) {
 		DEBUG(DEBUG_NAME_COMPRESSION, 2,
 		      (LOG_INFO, "query dname: %s, number: %lu, offset: %u\n",
-		       dname_to_string(domain_dname(domain), NULL),
+		       domain_to_string(domain),
 		       (unsigned long) domain->number,
 		       offset));
 		query_put_dname_offset(q, domain, offset);
@@ -165,7 +165,7 @@ query_cleanup(void *data)
 
 query_type *
 query_create(region_type *region, uint16_t *compressed_dname_offsets,
-	uint32_t compressed_dname_size)
+	size_t compressed_dname_size)
 {
 	query_type *query
 		= (query_type *) region_alloc_zero(region, sizeof(query_type));
@@ -212,7 +212,6 @@ query_reset(query_type *q, size_t maxlen, int is_tcp)
 	q->qtype = 0;
 	q->qclass = 0;
 	q->zone = NULL;
-	q->domain = NULL;
 	q->opcode = 0;
 	q->cname_count = 0;
 	q->delegation_domain = NULL;
@@ -363,8 +362,11 @@ process_tsig(struct query* q)
 		return NSD_RC_FORMAT;
 	if(q->tsig.status == TSIG_OK) {
 		if(!tsig_from_query(&q->tsig)) {
-			log_msg(LOG_ERR, "query: bad tsig (%s)",
-				tsig_error(q->tsig.error_code));
+			char a[128];
+			addr2str(&q->addr, a, sizeof(a));
+			log_msg(LOG_ERR, "query: bad tsig (%s) for key %s from %s",
+				tsig_error(q->tsig.error_code),
+				dname_to_string(q->tsig.key_name, NULL), a);
 			return NSD_RC_NOTAUTH;
 		}
 		buffer_set_limit(q->packet, q->tsig.position);
@@ -372,8 +374,10 @@ process_tsig(struct query* q)
 		tsig_prepare(&q->tsig);
 		tsig_update(&q->tsig, q->packet, buffer_limit(q->packet));
 		if(!tsig_verify(&q->tsig)) {
-			log_msg(LOG_ERR, "query: bad tsig signature for key %s",
-				dname_to_string(q->tsig.key->name, NULL));
+			char a[128];
+			addr2str(&q->addr, a, sizeof(a));
+			log_msg(LOG_ERR, "query: bad tsig signature for key %s from %s",
+				dname_to_string(q->tsig.key->name, NULL), a);
 			return NSD_RC_NOTAUTH;
 		}
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "query good tsig signature for %s",
@@ -412,7 +416,7 @@ answer_notify(struct nsd* nsd, struct query *query)
 		return query_error(query, rc);
 
 	/* check if it passes acl */
-	if((acl_num = acl_check_incoming(zone_opt->allow_notify, query,
+	if((acl_num = acl_check_incoming(zone_opt->pattern->allow_notify, query,
 		&why)) != -1)
 	{
 		sig_atomic_t mode = NSD_PASS_TO_XFRD;
@@ -421,13 +425,14 @@ answer_notify(struct nsd* nsd, struct query *query)
 		uint32_t acl_send = htonl(acl_num);
 		uint32_t acl_xfr;
 		size_t pos;
-		assert(why);
 
 		/* Find priority candidate for request XFR. -1 if no match */
 		acl_num_xfr = acl_check_incoming(
-			zone_opt->request_xfr, query, NULL);
+			zone_opt->pattern->request_xfr, query, NULL);
+
 		acl_xfr = htonl(acl_num_xfr);
 
+		assert(why);
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "got notify %s passed acl %s %s",
 			dname_to_string(query->qname, NULL),
 			why->ip_address_spec,
@@ -461,18 +466,19 @@ answer_notify(struct nsd* nsd, struct query *query)
 		pos = buffer_position(query->packet);
 		buffer_clear(query->packet);
 		buffer_set_position(query->packet, pos);
-		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd"));
+		if(verbosity >= 1) {
+			char address[128];
+			addr2str(&query->addr, address, sizeof(address));
+			VERBOSITY(2, (LOG_INFO, "notify for %s from %s",
+				dname_to_string(query->qname, NULL), address));
+		}
 		/* tsig is added in add_additional later (if needed) */
 		return QUERY_PROCESSED;
 	}
 
-	if (verbosity > 1) {
+	if (verbosity >= 1) {
 		char address[128];
-		if (addr2ip(query->addr, address, sizeof(address))) {
-			DEBUG(DEBUG_XFRD,1, (LOG_INFO, "addr2ip failed"));
-			strlcpy(address, "[unknown]", sizeof(address));
-		}
-
+		addr2str(&query->addr, address, sizeof(address));
 		VERBOSITY(1, (LOG_INFO, "notify for zone %s from client %s refused, %s%s",
 			dname_to_string(query->qname, NULL),
 			address,
@@ -548,7 +554,7 @@ find_covering_nsec(domain_type *closest_match,
 	assert(nsec_rrset);
 
 	/* loop away temporary created domains. For real ones it is &RBTREE_NULL */
-	while (closest_match->node.parent == NULL)
+	while (closest_match->rnode == NULL)
 		closest_match = closest_match->parent;
 	while (closest_match) {
 		*nsec_rrset = domain_find_rrset(closest_match, zone, TYPE_NSEC);
@@ -618,7 +624,8 @@ add_additional_rrsets(struct query *query, answer_type *answer,
 			domain_type *wildcard_child = domain_wildcard_child(match);
 			domain_type *temp = (domain_type *) region_alloc(
 				query->region, sizeof(domain_type));
-			memcpy(&temp->node, &additional->node, sizeof(rbnode_t));
+			temp->rnode = NULL;
+			temp->dname = additional->dname;
 			temp->number = additional->number;
 			temp->parent = match;
 			temp->wildcard_child_closest_match = temp;
@@ -693,7 +700,7 @@ add_rrset(struct query   *query,
    from_name is changes to to_name by the DNAME rr.
    DNAME rr is from src to dest.
    closest encloser encloses the to_name. */
-static uint32_t
+static size_t
 query_synthesize_cname(struct query* q, struct answer* answer, const dname_type* from_name,
 	const dname_type* to_name, domain_type* src, domain_type* to_closest_encloser,
 	domain_type** to_closest_match)
@@ -717,15 +724,15 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 			return 0;
 		newdom->is_existing = 1;
 		newdom->parent = lastparent;
-		newdom->node.key = dname_partial_copy(q->region,
+		newdom->dname
+			= dname_partial_copy(q->region,
 			from_name, domain_dname(src)->label_count + i + 1);
 		if(dname_compare(domain_dname(newdom), q->qname) == 0) {
 			/* 0 good for query name, otherwise new number */
 			newdom->number = 0;
 		}
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "created temp domain src %d. %s nr %d", i,
-			dname_to_string(domain_dname(newdom), NULL),
-			newdom->number));
+			domain_to_string(newdom), (int)newdom->number));
 		lastparent = newdom;
 	}
 	cname_domain = lastparent;
@@ -740,11 +747,11 @@ query_synthesize_cname(struct query* q, struct answer* answer, const dname_type*
 			return 0;
 		newdom->is_existing = 0;
 		newdom->parent = lastparent;
-		newdom->node.key = dname_partial_copy(q->region,
+		newdom->dname
+			= dname_partial_copy(q->region,
 			to_name, domain_dname(to_closest_encloser)->label_count + i + 1);
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "created temp domain dest %d. %s nr %d", i,
-			dname_to_string(domain_dname(newdom), NULL),
-			newdom->number));
+			domain_to_string(newdom), (int)newdom->number));
 		lastparent = newdom;
 	}
 	cname_dest = lastparent;
@@ -803,7 +810,7 @@ answer_delegation(query_type *query, answer_type *answer)
 			add_rrset(query, answer, AUTHORITY_SECTION,
 				  query->delegation_domain, rrset);
 #ifdef NSEC3
-		} else if (query->zone->nsec3_soa_rr) {
+		} else if (query->zone->nsec3_param) {
 			nsec3_answer_delegation(query, answer);
 #endif
 		} else if ((rrset = domain_find_rrset(query->delegation_domain, query->zone, TYPE_NSEC))) {
@@ -811,7 +818,6 @@ answer_delegation(query_type *query, answer_type *answer)
 				  query->delegation_domain, rrset);
 		}
 	}
-	query->domain = query->delegation_domain;
 }
 
 
@@ -821,8 +827,6 @@ answer_delegation(query_type *query, answer_type *answer)
 static void
 answer_soa(struct query *query, answer_type *answer)
 {
-	query->domain = query->zone->apex;
-
 	if (query->qclass != CLASS_ANY) {
 		add_rrset(query, answer,
 			  AUTHORITY_SECTION,
@@ -849,7 +853,7 @@ answer_nodata(struct query *query, answer_type *answer, domain_type *original)
 	}
 
 #ifdef NSEC3
-	if (query->edns.dnssec_ok && query->zone->nsec3_soa_rr) {
+	if (query->edns.dnssec_ok && query->zone->nsec3_param) {
 		nsec3_answer_nodata(query, answer, original);
 	} else
 #endif
@@ -939,16 +943,12 @@ answer_domain(struct nsd* nsd, struct query *q, answer_type *answer,
 					     domain_dname(closest_match));
 			q->zone = origzone;
 		}
-		/* example 6.2.7 shows no NS-set from zone in auth (RFC1034) */
-		q->domain = domain;
 		return;
 	} else {
 		answer_nodata(q, answer, original);
 		return;
 	}
 
-	q->domain = domain;
-
 	if (q->qclass != CLASS_ANY && q->zone->ns_rrset && answer_needs_ns(q)) {
 		add_rrset(q, answer, OPTIONAL_AUTHORITY_SECTION, q->zone->apex,
 			  q->zone->ns_rrset);
@@ -969,7 +969,7 @@ static void
 answer_authoritative(struct nsd   *nsd,
 		     struct query *q,
 		     answer_type  *answer,
-		     uint32_t      domain_number,
+		     size_t        domain_number,
 		     int           exact,
 		     domain_type  *closest_match,
 		     domain_type  *closest_encloser,
@@ -999,16 +999,16 @@ answer_authoritative(struct nsd   *nsd,
 			name = domain_dname(closest_match);
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "expanding DNAME for q=%s", dname_to_string(name, NULL)));
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->src is %s",
-			dname_to_string(domain_dname(closest_encloser), NULL)));
+			domain_to_string(closest_encloser)));
 		DEBUG(DEBUG_QUERY,2, (LOG_INFO, "->dest is %s",
-			dname_to_string(domain_dname(dest), NULL)));
+			domain_to_string(dest)));
 		/* if the DNAME set is not added we have a loop, do not follow */
 		added = add_rrset(q, answer, ANSWER_SECTION, closest_encloser, rrset);
 		if(added) {
 			domain_type* src = closest_encloser;
 			const dname_type* newname = dname_replace(q->region, name,
 				domain_dname(src), domain_dname(dest));
-			uint32_t newnum = 0;
+			size_t newnum = 0;
 			zone_type* origzone = q->zone;
 			++q->cname_count;
 			if(!newname) { /* newname too long */
@@ -1047,23 +1047,26 @@ answer_authoritative(struct nsd   *nsd,
 
 		match = (domain_type *) region_alloc(q->region,
 						     sizeof(domain_type));
-		memcpy(&match->node, &wildcard_child->node, sizeof(rbnode_t));
+		match->rnode = NULL;
+		match->dname = wildcard_child->dname;
 		match->parent = closest_encloser;
 		match->wildcard_child_closest_match = match;
 		match->number = domain_number;
 		match->rrsets = wildcard_child->rrsets;
 		match->is_existing = wildcard_child->is_existing;
 #ifdef NSEC3
-		match->nsec3_cover = wildcard_child->nsec3_cover;
-#ifdef FULL_PREHASH
+		match->nsec3 = wildcard_child->nsec3;
+		/* copy over these entries:
 		match->nsec3_is_exact = wildcard_child->nsec3_is_exact;
+		match->nsec3_cover = wildcard_child->nsec3_cover;
 		match->nsec3_wcard_child_cover = wildcard_child->nsec3_wcard_child_cover;
 		match->nsec3_ds_parent_is_exact = wildcard_child->nsec3_ds_parent_is_exact;
 		match->nsec3_ds_parent_cover = wildcard_child->nsec3_ds_parent_cover;
-#endif
-		if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
+		*/
+
+		if (q->edns.dnssec_ok && q->zone->nsec3_param) {
 			/* Only add nsec3 wildcard data when do bit is set */
-			nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
+			nsec3_answer_wildcard(q, answer, wildcard_child, qname);
 		}
 #endif
 
@@ -1080,9 +1083,9 @@ answer_authoritative(struct nsd   *nsd,
 
 	/* Authorative zone.  */
 #ifdef NSEC3
-	if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
+	if (q->edns.dnssec_ok && q->zone->nsec3_param) {
 		nsec3_answer_authoritative(&match, q, answer,
-			closest_encloser, nsd->db, qname);
+			closest_encloser, qname);
 	} else
 #endif
 	if (q->edns.dnssec_ok && zone_is_secure(q->zone)) {
@@ -1131,11 +1134,18 @@ answer_authoritative(struct nsd   *nsd,
  */
 static void
 answer_lookup_zone(struct nsd *nsd, struct query *q, answer_type *answer,
-	uint32_t domain_number, int exact, domain_type *closest_match,
+	size_t domain_number, int exact, domain_type *closest_match,
 	domain_type *closest_encloser, const dname_type *qname)
 {
-	q->zone = domain_find_zone(closest_encloser);
+	q->zone = domain_find_zone(nsd->db, closest_encloser);
 	if (!q->zone) {
+		/* no zone for this */
+		if(q->cname_count == 0)
+			RCODE_SET(q->packet, RCODE_REFUSE);
+		return;
+	}
+	if(!q->zone->apex || !q->zone->soa_rrset) {
+		/* zone is configured but not loaded */
 		if(q->cname_count == 0)
 			RCODE_SET(q->packet, RCODE_SERVFAIL);
 		return;
@@ -1157,8 +1167,8 @@ answer_lookup_zone(struct nsd *nsd, struct query *q, answer_type *answer,
 	}
 
 	/* see if the zone has expired (for secondary zones) */
-	if(q->zone && q->zone->opts && zone_is_slave(q->zone->opts)
-		&& !q->zone->is_ok) {
+	if(q->zone && q->zone->opts && q->zone->opts->pattern &&
+		q->zone->opts->pattern->request_xfr != 0 && !q->zone->is_ok) {
 		if(q->cname_count == 0)
 			RCODE_SET(q->packet, RCODE_SERVFAIL);
 		return;
@@ -1218,18 +1228,9 @@ answer_query(struct nsd *nsd, struct query *q)
 		return;
 	}
 
-	q->domain = closest_encloser;
 	answer_lookup_zone(nsd, q, &answer, 0, exact, closest_match,
 		closest_encloser, q->qname);
 
-#ifdef USE_ZONE_STATS
-	if (q->zone) {
-		ZTATUP2(q->zone, opcode, q->opcode);
-		ZTATUP2(q->zone, qtype, q->qtype);
-		ZTATUP2(q->zone, opcode, q->qclass);
-	}
-#endif
-
 	offset = dname_label_offsets(q->qname)[domain_dname(closest_encloser)->label_count - 1] + QHEADERSZ;
 	query_add_compression_domain(q, closest_encloser, offset);
 	encode_answer(q, &answer);
@@ -1323,7 +1324,7 @@ query_process(query_type *q, nsd_type *nsd)
 
 	arcount = ARCOUNT(q->packet);
 	if (arcount > 0) {
-		/* According to RFC 6891:
+		/* According to draft-ietf-dnsext-rfc2671bis-edns0-10:
 		 * "The placement flexibility for the OPT RR does not
 		 * override the need for the TSIG or SIG(0) RRs to be
 		 * the last in the additional section whenever they are
@@ -1432,11 +1433,6 @@ query_add_optional(query_type *q, nsd_type *nsd)
 		}
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
 		STATUP(nsd, edns);
-#ifdef USE_ZONE_STATS
-		if (q->zone) {
-			ZTATUP(q->zone, edns);
-		}
-#endif
 		break;
 	case EDNS_ERROR:
 		if (q->edns.dnssec_ok)	edns->error[7] = 0x80;
@@ -1445,11 +1441,6 @@ query_add_optional(query_type *q, nsd_type *nsd)
 		buffer_write(q->packet, edns->rdata_none, OPT_RDATA);
 		ARCOUNT_SET(q->packet, ARCOUNT(q->packet) + 1);
 		STATUP(nsd, ednserr);
-#ifdef USE_ZONE_STATS
-		if (q->zone) {
-			ZTATUP(q->zone, ednserr);
-		}
-#endif
 		break;
 	}
 
